@@ -16,8 +16,8 @@
 
 import abc
 import copy
+import functools
 
-from oslo_cache import core as oslo_cache
 from oslo_config import cfg
 from oslo_log import log
 from oslo_log import versionutils
@@ -43,7 +43,7 @@ MEMOIZE = cache.get_memoization_decorator(group='role')
 # This builds a discrete cache region dedicated to role assignments computed
 # for a given user + project/domain pair. Any write operation to add or remove
 # any role assignment should invalidate this entire cache region.
-COMPUTED_ASSIGNMENTS_REGION = oslo_cache.create_region()
+COMPUTED_ASSIGNMENTS_REGION = cache.create_region(name='computed assignments')
 MEMOIZE_COMPUTED_ASSIGNMENTS = cache.get_memoization_decorator(
     group='role',
     region=COMPUTED_ASSIGNMENTS_REGION)
@@ -1495,6 +1495,31 @@ class RoleManager(manager.Manager):
         elif not isinstance(self.driver, RoleDriverV9):
             raise exception.UnsupportedDriverVersion(driver=role_driver)
 
+    def _append_null_domain_id(f):
+        """Append a domain_id field to a role dict if it is not already there.
+
+        When caching is turned on, upgrading from liberty to
+        mitaka or master causes tokens to fail to be issued for the
+        time-to-live of the cache. This is because as part of the token
+        issuance the token's role is looked up, and the cached version of the
+        role immediately after upgrade does not have a domain_id field, even
+        though that column was successfully added to the role database. This
+        decorator artificially adds a null domain_id value to the role
+        reference so that the cached value acts like the updated schema.
+
+        Note: This decorator must appear before the @MEMOIZE decorator
+        because it operates on the cached value returned by the MEMOIZE
+        function.
+        """
+        @functools.wraps(f)
+        def wrapper(self, *args, **kwargs):
+            ref = f(self, *args, **kwargs)
+            if 'domain_id' not in ref:
+                ref['domain_id'] = None
+            return ref
+        return wrapper
+
+    @_append_null_domain_id
     @MEMOIZE
     def get_role(self, role_id):
         return self.driver.get_role(role_id)
