@@ -18,7 +18,6 @@
 import abc
 import itertools
 
-from oslo_cache import core as oslo_cache
 from oslo_config import cfg
 from oslo_log import log
 import six
@@ -49,7 +48,7 @@ MEMOIZE = cache.get_memoization_decorator(group='catalog')
 # computed for a given user + project pair. Any write operation to create,
 # modify or delete elements of the service catalog should invalidate this
 # entire cache region.
-COMPUTED_CATALOG_REGION = oslo_cache.create_region()
+COMPUTED_CATALOG_REGION = cache.create_region(name='computed catalog region')
 MEMOIZE_COMPUTED_CATALOG = cache.get_memoization_decorator(
     group='catalog',
     region=COMPUTED_CATALOG_REGION)
