@@ -165,6 +165,18 @@ class Federation(core.FederationDriverV9):
 
     _CONFLICT_LOG_MSG = 'Conflict %(conflict_type)s: %(details)s'
 
+    def _handle_idp_conflict(self, e):
+        conflict_type = 'identity_provider'
+        details = six.text_type(e)
+        LOG.debug(self._CONFLICT_LOG_MSG, {'conflict_type': conflict_type,
+                                           'details': details})
+        if 'remote_id' in details:
+            msg = _('Duplicate remote ID: %s')
+        else:
+            msg = _('Duplicate entry: %s')
+        msg = msg % e.value
+        raise exception.Conflict(type=conflict_type, details=msg)
+
     # Identity Provider CRUD
     def create_idp(self, idp_id, idp):
         idp['id'] = idp_id
@@ -174,16 +186,7 @@ class Federation(core.FederationDriverV9):
                 session.add(idp_ref)
                 return idp_ref.to_dict()
         except sql.DBDuplicateEntry as e:
-            conflict_type = 'identity_provider'
-            details = six.text_type(e)
-            LOG.debug(self._CONFLICT_LOG_MSG, {'conflict_type': conflict_type,
-                                               'details': details})
-            if 'remote_id' in details:
-                msg = _('Duplicate remote ID: %s')
-            else:
-                msg = _('Duplicate entry: %s')
-            msg = msg % e.value
-            raise exception.Conflict(type=conflict_type, details=msg)
+            self._handle_idp_conflict(e)
 
     def delete_idp(self, idp_id):
         with sql.session_for_write() as session:
@@ -223,14 +226,17 @@ class Federation(core.FederationDriverV9):
             return ref.to_dict()
 
     def update_idp(self, idp_id, idp):
-        with sql.session_for_write() as session:
-            idp_ref = self._get_idp(session, idp_id)
-            old_idp = idp_ref.to_dict()
-            old_idp.update(idp)
-            new_idp = IdentityProviderModel.from_dict(old_idp)
-            for attr in IdentityProviderModel.mutable_attributes:
-                setattr(idp_ref, attr, getattr(new_idp, attr))
-            return idp_ref.to_dict()
+        try:
+            with sql.session_for_write() as session:
+                idp_ref = self._get_idp(session, idp_id)
+                old_idp = idp_ref.to_dict()
+                old_idp.update(idp)
+                new_idp = IdentityProviderModel.from_dict(old_idp)
+                for attr in IdentityProviderModel.mutable_attributes:
+                    setattr(idp_ref, attr, getattr(new_idp, attr))
+                return idp_ref.to_dict()
+        except sql.DBDuplicateEntry as e:
+            self._handle_idp_conflict(e)
 
     # Protocol CRUD
     def _get_protocol(self, session, idp_id, protocol_id):
