@@ -44,18 +44,18 @@ def init():
 def enforce(credentials, action, target, do_raise=True):
     """Verifies that the action is valid on the target in this context.
 
-       :param credentials: user credentials
-       :param action: string representing the action to be checked, which
-                      should be colon separated for clarity.
-       :param target: dictionary representing the object of the action
-                      for object creation this should be a dictionary
-                      representing the location of the object e.g.
-                      {'project_id': object.project_id}
-       :raises: `exception.Forbidden` if verification fails.
+    :param credentials: user credentials
+    :param action: string representing the action to be checked, which should
+                   be colon separated for clarity.
+    :param target: dictionary representing the object of the action for object
+                   creation this should be a dictionary representing the
+                   location of the object e.g. {'project_id':
+                   object.project_id}
+    :raises keystone.exception.Forbidden: If verification fails.
 
-       Actions should be colon separated for clarity. For example:
+    Actions should be colon separated for clarity. For example:
 
-        * identity:list_users
+    * identity:list_users
 
     """
     init()
