@@ -59,12 +59,12 @@ class DomainConfig(resource.DomainConfigDriverV8):
     @sql.handle_conflicts(conflict_type='domain_config')
     def create_config_option(self, domain_id, group, option, value,
                              sensitive=False):
-        with sql.transaction() as session:
+        with sql.session_for_write() as session:
             config_table = self.choose_table(sensitive)
             ref = config_table(domain_id=domain_id, group=group,
                                option=option, value=value)
             session.add(ref)
-        return ref.to_dict()
+            return ref.to_dict()
 
     def _get_config_option(self, session, domain_id, group, option, sensitive):
         try:
@@ -80,14 +80,14 @@ class DomainConfig(resource.DomainConfigDriverV8):
         return ref
 
     def get_config_option(self, domain_id, group, option, sensitive=False):
-        with sql.transaction() as session:
+        with sql.session_for_read() as session:
             ref = self._get_config_option(session, domain_id, group, option,
                                           sensitive)
-        return ref.to_dict()
+            return ref.to_dict()
 
     def list_config_options(self, domain_id, group=None, option=None,
                             sensitive=False):
-        with sql.transaction() as session:
+        with sql.session_for_read() as session:
             config_table = self.choose_table(sensitive)
             query = session.query(config_table)
             query = query.filter_by(domain_id=domain_id)
@@ -99,11 +99,11 @@ class DomainConfig(resource.DomainConfigDriverV8):
 
     def update_config_option(self, domain_id, group, option, value,
                              sensitive=False):
-        with sql.transaction() as session:
+        with sql.session_for_write() as session:
             ref = self._get_config_option(session, domain_id, group, option,
                                           sensitive)
             ref.value = value
-        return ref.to_dict()
+            return ref.to_dict()
 
     def delete_config_options(self, domain_id, group=None, option=None,
                               sensitive=False):
@@ -114,7 +114,7 @@ class DomainConfig(resource.DomainConfigDriverV8):
         if there was nothing to delete.
 
         """
-        with sql.transaction() as session:
+        with sql.session_for_write() as session:
             config_table = self.choose_table(sensitive)
             query = session.query(config_table)
             query = query.filter_by(domain_id=domain_id)
@@ -126,25 +126,25 @@ class DomainConfig(resource.DomainConfigDriverV8):
 
     def obtain_registration(self, domain_id, type):
         try:
-            with sql.transaction() as session:
+            with sql.session_for_write() as session:
                 ref = ConfigRegister(type=type, domain_id=domain_id)
                 session.add(ref)
             return True
-        except sql.DBDuplicateEntry:
+        except sql.DBDuplicateEntry:  # nosec
+            # Continue on and return False to indicate failure.
             pass
         return False
 
     def read_registration(self, type):
-        with sql.transaction() as session:
+        with sql.session_for_read() as session:
             ref = session.query(ConfigRegister).get(type)
             if not ref:
                 raise exception.ConfigRegistrationNotFound()
-        return ref.domain_id
+            return ref.domain_id
 
     def release_registration(self, domain_id, type=None):
         """Silently delete anything registered for the domain specified."""
-
-        with sql.transaction() as session:
+        with sql.session_for_write() as session:
             query = session.query(ConfigRegister)
             if type:
                 query = query.filter_by(type=type)
