@@ -607,16 +607,70 @@ class MappingRuleEngineTests(unit.BaseTestCase):
         self.assertEqual('Developer',
                          mapped_properties['group_names'][0]['name'])
 
-    def test_mapping_with_incorrect_local_keys(self):
+    def test_mapping_validation_with_incorrect_local_keys(self):
         mapping = mapping_fixtures.MAPPING_BAD_LOCAL_SETUP
         self.assertRaises(exception.ValidationError,
                           mapping_utils.validate_mapping_structure,
                           mapping)
 
-    def test_mapping_with_group_name_and_domain(self):
+    def test_mapping_validation_with_user_name_and_domain_name(self):
+        mapping = mapping_fixtures.MAPPING_WITH_USERNAME_AND_DOMAINNAME
+        mapping_utils.validate_mapping_structure(mapping)
+
+    def test_mapping_validation_with_user_name_and_domain_id(self):
+        mapping = mapping_fixtures.MAPPING_WITH_USERNAME_AND_DOMAINID
+        mapping_utils.validate_mapping_structure(mapping)
+
+    def test_mapping_validation_with_user_id_and_domain_id(self):
+        mapping = mapping_fixtures.MAPPING_WITH_USERID_AND_DOMAINID
+        mapping_utils.validate_mapping_structure(mapping)
+
+    def test_mapping_validation_with_group_name_and_domain(self):
         mapping = mapping_fixtures.MAPPING_GROUP_NAMES
         mapping_utils.validate_mapping_structure(mapping)
 
+    def test_mapping_validation_bad_domain(self):
+        mapping = mapping_fixtures.MAPPING_BAD_DOMAIN
+        self.assertRaises(exception.ValidationError,
+                          mapping_utils.validate_mapping_structure,
+                          mapping)
+
+    def test_mapping_validation_bad_group(self):
+        mapping = mapping_fixtures.MAPPING_BAD_GROUP
+        self.assertRaises(exception.ValidationError,
+                          mapping_utils.validate_mapping_structure,
+                          mapping)
+
+    def test_mapping_validation_with_group_name_without_domain(self):
+        mapping = mapping_fixtures.MAPPING_GROUP_NAME_WITHOUT_DOMAIN
+        self.assertRaises(exception.ValidationError,
+                          mapping_utils.validate_mapping_structure,
+                          mapping)
+
+    def test_mapping_validation_with_group_id_and_domain(self):
+        mapping = mapping_fixtures.MAPPING_GROUP_ID_WITH_DOMAIN
+        self.assertRaises(exception.ValidationError,
+                          mapping_utils.validate_mapping_structure,
+                          mapping)
+
+    def test_mapping_validation_no_local(self):
+        mapping = mapping_fixtures.MAPPING_MISSING_LOCAL
+        self.assertRaises(exception.ValidationError,
+                          mapping_utils.validate_mapping_structure,
+                          mapping)
+
+    def test_mapping_validataion_no_remote(self):
+        mapping = mapping_fixtures.MAPPING_NO_REMOTE
+        self.assertRaises(exception.ValidationError,
+                          mapping_utils.validate_mapping_structure,
+                          mapping)
+
+    def test_mapping_validation_no_type(self):
+        mapping = mapping_fixtures.MAPPING_MISSING_TYPE
+        self.assertRaises(exception.ValidationError,
+                          mapping_utils.validate_mapping_structure,
+                          mapping)
+
     def test_type_not_in_assertion(self):
         """Test that if the remote "type" is not in the assertion it fails."""
         mapping = mapping_fixtures.MAPPING_GROUPS_WHITELIST_PASS_THROUGH
@@ -626,6 +680,24 @@ class MappingRuleEngineTests(unit.BaseTestCase):
                           rp.process,
                           assertion)
 
+    def test_rule_engine_groups_mapping_only_one_group(self):
+        """Test mapping engine when groups is explicitly set.
+
+        If the groups list has only one group,
+        test if the transformation is done correctly
+
+        """
+        mapping = mapping_fixtures.MAPPING_GROUPS_WITH_EMAIL
+        assertion = mapping_fixtures.GROUPS_ASSERTION_ONLY_ONE_GROUP
+        rp = mapping_utils.RuleProcessor(FAKE_MAPPING_ID, mapping['rules'])
+        mapped_properties = rp.process(assertion)
+        self.assertIsNotNone(mapped_properties)
+        self.assertEqual('jsmith', mapped_properties['user']['name'])
+        self.assertEqual('jill@example.com',
+                         mapped_properties['user']['email'])
+        self.assertEqual('ALL USERS',
+                         mapped_properties['group_names'][0]['name'])
+
     def test_rule_engine_group_ids_mapping_whitelist(self):
         """Test mapping engine when group_ids is explicitly set
 
