@@ -21,8 +21,10 @@ from oslo_config import cfg
 from six.moves import range
 from testtools import matchers
 
+from keystone.auth import controllers
 from keystone.cmd import cli
 from keystone.common import dependency
+from keystone import exception
 from keystone.i18n import _
 from keystone import resource
 from keystone.tests import unit
@@ -111,12 +113,83 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
                 self.assertEqual(svc['id'], endpoint['service_id'])
                 self.assertEqual(interface, endpoint['interface'])
 
-    def test_bootstrap_is_idempotent(self):
-        # NOTE(morganfainberg): Ensure we can run bootstrap multiple times
-        # without erroring.
+    def test_bootstrap_is_idempotent_when_password_does_not_change(self):
+        # NOTE(morganfainberg): Ensure we can run bootstrap with the same
+        # configuration multiple times without erroring.
         bootstrap = cli.BootStrap()
         self._do_test_bootstrap(bootstrap)
+        v3_token_controller = controllers.Auth()
+        v3_password_data = {
+            'identity': {
+                "methods": ["password"],
+                "password": {
+                    "user": {
+                        "name": bootstrap.username,
+                        "password": bootstrap.password,
+                        "domain": {
+                            "id": CONF.identity.default_domain_id
+                        }
+                    }
+                }
+            }
+        }
+
+        context = {'environment': {}, 'query_string': {}}
+        auth_response = v3_token_controller.authenticate_for_token(
+            context, v3_password_data)
+        token = auth_response.headers.get('X-Subject-Token')
         self._do_test_bootstrap(bootstrap)
+        # build validation request
+        context = {'is_admin': True,
+                   'subject_token_id': token,
+                   'token_id': token,
+                   'query_string': {},
+                   'environment': {}}
+        # Make sure the token we authenticate for is still valid.
+        v3_token_controller.validate_token(context)
+
+    def test_bootstrap_is_not_idempotent_when_password_does_change(self):
+        # NOTE(lbragstad): Ensure bootstrap isn't idempotent when run with
+        # different arguments or configuration values.
+        bootstrap = cli.BootStrap()
+        self._do_test_bootstrap(bootstrap)
+        v3_token_controller = controllers.Auth()
+        v3_password_data = {
+            'identity': {
+                "methods": ["password"],
+                "password": {
+                    "user": {
+                        "name": bootstrap.username,
+                        "password": bootstrap.password,
+                        "domain": {
+                            "id": CONF.identity.default_domain_id
+                        }
+                    }
+                }
+            }
+        }
+        context = {'environment': {}, 'query_string': {}}
+        auth_response = v3_token_controller.authenticate_for_token(
+            context, v3_password_data)
+        token = auth_response.headers.get('X-Subject-Token')
+        os.environ['OS_BOOTSTRAP_PASSWORD'] = uuid.uuid4().hex
+        self._do_test_bootstrap(bootstrap)
+        # build validation request
+        context = {'is_admin': True,
+                   'subject_token_id': token,
+                   'token_id': token,
+                   'query_string': {},
+                   'environment': {}}
+        # Since the user account was recovered with a different password, we
+        # shouldn't be able to validate this token. Bootstrap should have
+        # persisted a revocation event because the user's password was updated.
+        # Since this token was obtained using the original password, it should
+        # now be invalid.
+        self.assertRaises(
+            exception.TokenNotFound,
+            v3_token_controller.validate_token,
+            context
+        )
 
     def test_bootstrap_recovers_user(self):
         bootstrap = cli.BootStrap()
