@@ -52,15 +52,6 @@ class PolicyFileTestCase(unit.TestCase):
         self.assertRaises(exception.ForbiddenAction, rules.enforce,
                           empty_credentials, action, self.target)
 
-    def test_invalid_policy_raises_error(self):
-        action = "example:test"
-        empty_credentials = {}
-        invalid_json = '{"example:test": [],}'
-        with open(self.tmpfilename, "w") as policyfile:
-            policyfile.write(invalid_json)
-        self.assertRaises(ValueError, rules.enforce,
-                          empty_credentials, action, self.target)
-
 
 class PolicyTestCase(unit.TestCase):
     def setUp(self):
